(function (window) {
    function isAndroid() {
        var ua = navigator.userAgent,
            _isAndroid = ua.indexOf('Android') > -1 || ua.indexOf('Linux') > -1;
        return _isAndroid;
    }

    function isIOS() {
        return !!navigator.userAgent.match(/(i[^;]+\;(U;)? CPU.+Mac OS X)/);
    }

    function isPad() {
        return navigator.userAgent.toLowerCase().match(/iPad/i) == "ipad";
    }

    function isWinPad() {
        return navigator.userAgent.indexOf("Windows NT") >= 0;
    }

    var AppBridge = {};
    AppBridge.toast = {};  // toast相关
    AppBridge.image = {};  // 图片相关

    /**
     * 显示toast
     * @param  {Object}   options  参数对象
     * @param  {Function} callback 回调
     */
    AppBridge.toast.show = function (options, callback) {
        var defaults = {
            message: '',       // 消息内容
            messageColor: 'ffffff', // 消息颜色
            backgroundColor: '5eb95e', // 背景颜色
            duration: 2.0,      // 持续时间
            extras: {},       // 额外内容 callback中原样返回
        };
        options = mergeObj(defaults, options);
        callHandler('appBridgeToastShow', options, callback);
    }

    /**
     * 隐藏toast
     * @param  {Object}   options  参数对象
     * @param  {Function} callback 回调
     */
    AppBridge.toast.hide = function (options, callback) {
        var defaults = {
            extras: {}, // 额外内容 callback中原样返回
        };
        options = mergeObj(defaults, options);
        callHandler('appBridgeToastHide', options, callback);
    }
    // 登录成功
    AppBridge.loginSuccess = function(options, callback) {
        callHandler('appBridgeLoginSuccess', options, callback);
    }

    // 退出登录
    AppBridge.Logout = function(options, callback) {
        callHandler('appBridgeLogout', options, callback);
    }

    // 上传图片
    AppBridge.uploadImages = function (options, callback) {
        var defaults = {
            multi: true,    //是否多选
            max: 9          //最大可选图片数量
        };
        options = mergeObj(defaults, options);
        callHandler('appBridgeUploadImages', options, callback);
    }

    /**
     * 打开新链接
     * 1.id与url同时存在 使用对应id页打开url
     * 2.仅id存在 打开原生语言写的不需要url的页
     * 3.仅url存在 使用普通浏览器页打开url
     * @param  {Object}   options  参数对象
     * @param  {Function} callback 回调
     */
    AppBridge.commonUrlOpen = function (options, callback) {
        //通用页面打开
        callHandler('appBridgeCommonUrlOpen', options, callback);
    }

    //单页关闭
    AppBridge.close = function (fnname, param) {
        var defaults = {
            fnname: fnname,     // 对应页id(不区分大小写)DiseaseDetail
            param: JSON.stringify(param),   // 额外内容 callback中原样返回
            toRoot: false,      //是否返回首页
            pageCloseNum: 1,    //关闭窗口层级
            isRefresh: false     //是否重新加载页面
        };
        callHandler('appBridgeClose', defaults, function () { });
    }
    
    /**
     * 图片浏览器
     * @param  {Object}   options  参数对象
     * @param  {Function} callback 回调
     */
    AppBridge.image.browser = function (options, callback) {
        var defaults = {
            urls: [], // 图片链接数组
            titles: [], // 对应标题
            index: 0,  // 当前索引
            extras: {}, // 额外内容 callback中原样返回
        };
        options = mergeObj(defaults, options);
        callHandler('appBridgeImageBrowser', options, callback);
    }

    AppBridge.camera = function (callback) {
        var defaults = {};
        callHandler('appBridgeCameraPhotoGet', defaults, callback);
    }

    //app扫一扫功能
    AppBridge.nativeScan = function (param, callback) {
        var defaults = {
            barCode: 1,     //是否识别条形码
            qrCode: 1       //是否识别二维码
        };
        param = mergeObj(defaults, param);
        callHandler('appBridgeScan', param, callback);
    }

    //app存储数据
    AppBridge.setData = function (param, callback) {
        callHandler('appBridgeSetData', param, callback);
    }
    //app取出数据
    AppBridge.getData = function (param, callback) {
        callHandler('appBridgeGetData', param, callback);
    }

    //去评分
    AppBridge.goToAppItunes = function (param, callback) {
        callHandler('appBridgeGoToAppItunes', "", function () { });
    }

    //获取系统信息
    AppBridge.getSystemVersion = function (callback) {
        callHandler('appBridgeVersion', {}, callback);
    }

    AppBridge.refresh = function(path,param){
        var href = location.origin + location.pathname + "?f=app";
        href += "&p=" + path;
        if (param) {
            for(var key in param) {
                href += "&" + key + "=" + param[key];
            }
        }
        location.href = href;
    }

    //分享
    AppBridge.share = function (param, callback) {
        var defaults = {
            title: "海纳云大学", // 分享标题
            desc: "", // 分享描述
            link: "", // 分享链接
            imgUrl: "", // 分享图标
        };
        param = mergeObj(defaults, param);
        callHandler('appBridgeShareSDK', param, function () { });
    }

    //获取缓存
    AppBridge.clearCache = function (param, callback) {
        callHandler('appBridgeClearCache', '', callback);
    }

    /**
    * 对象合并obj2覆盖obj1
    * @param  {Object} obj1 对象1
    * @param  {Object} obj2 对象2
    * @return {Object}      合并后对象
    */
    function mergeObj(obj1, obj2) {
        if (!obj2) {
            return obj1;
        }
        for (var key in obj2) {
            obj1[key] = obj2[key];
        }
        return obj1;
    }

    /**
     * js-app桥接
     * @param  {Function} callback
     */
    function connectWebViewJavascriptBridge(callback) {
        if (window.WebViewJavascriptBridge) {
            return callback(WebViewJavascriptBridge);
        }

        /*IOS或者ipad*/
        if (isIOS() || isPad()) {
            if (window.WVJBCallbacks) {
                return window.WVJBCallbacks.push(callback);
            }
            window.WVJBCallbacks = [callback];
            var WVJBIframe = document.createElement('iframe');
            WVJBIframe.style.display = 'none';
            WVJBIframe.src = 'wvjbscheme://__BRIDGE_LOADED__';
            document.documentElement.appendChild(WVJBIframe);
            setTimeout(function () {
                document.documentElement.removeChild(WVJBIframe)
            }, 0);
        }
        else {
            console.log("Android");
            document.addEventListener(
                'WebViewJavascriptBridgeReady'
                , function () {
                    callback(WebViewJavascriptBridge)
                },
                false
            );
        }
    }

    /**
     * 调用app方法
     * @param  {String}   handlerName 对应app方法名
     * @param  {Object}   options     参数对象
     * @param  {Function} callback    调用成功回调
     */
    function callHandler(handlerName, options, callback) {
        log && log('callHandler:' + handlerName, options);
        console.log('callHandler:' + handlerName, options);

        connectWebViewJavascriptBridge(function (bridge) {
            bridge.callHandler(handlerName, options, function (result) {
                log("result:" + result);
                if (isIOS() || isPad()) {
                    result = result || {};
                }
                else {
                    result = result || "{}";
                    result = JSON.parse(result) || {};
                }

                log && log('callback:' + handlerName, result)
                console.log('callback:' + handlerName, result);
                callback && callback(result);
            });
        });
    }

    AppBridge.registerHandler = registerHandler;
    /**
     * 提供给app调用的方法
     * @param  {String}   handlerName 方法名
     */
    function registerHandler(handlerName, pageCallback) {
        log && log('registerHandler:' + handlerName);
        console.log('registerHandler:' + handlerName);

        connectWebViewJavascriptBridge(function (bridge) {
            bridge.registerHandler(handlerName, function (data, appCallback) {
                if (isIOS() || isPad()) {
                    data = data || {};
                }
                else {
                    data = data || "{}";
                    data = JSON.parse(data) || {};
                }
                log && log('callback:' + handlerName, data)
                console.log('callback:' + handlerName, data);
                if (pageCallback) {
                    pageCallback(data, appCallback);
                }
                else {
                    appCallback && appCallback(data);
                }
            });
        });
    }

    /*初始化js-bridge*/
    connectWebViewJavascriptBridge(function (bridge) {
        if (bridge && bridge.init) {
            bridge.init(function () {
                initRegisterHanger();
            });
        }
        initRegisterHanger();
    })

    function initRegisterHanger() {
        //弹窗选择角色
        AppBridge.registerHandler("appBridgeChangeLayout", function (data) {
            //获取当前页面的control
            var $scope = me.control().$scope;
            $scope.isCard = !$scope.isCard
            //触发当前页面的apply
            $scope.$apply();
        });

        AppBridge.registerHandler("appbridgePageback", function () {
            //检查当前ctrl是否有appbridgePageback
            if (me.control() && me.control().appbridgePageback)
                me.control().appbridgePageback();
        });

        //页面进入，app调用通知页面进入
        AppBridge.registerHandler("appbridgePageEnter", function () {
            //检查当前ctrl是否有appbridgePageback
            if (me.control() && me.control().appbridgePageEnter)
                me.control().appbridgePageEnter();
        });

        //页面离开，app调用通知页面离开
        AppBridge.registerHandler("appbridgePageOut", function () {
            //检查当前ctrl是否有appbridgePageback
            if (me.control() && me.control().appbridgePageOut)
                me.control().appbridgePageOut();
        });
    }

    window.AppBridge = AppBridge;
})(window);

function log(message, data) {
    var $log = $('#log');
    var uniqueId = $log.find('div').length + 1;
    var $ele = $('<div>' + uniqueId + '. ' + message + ':<br/>' + JSON.stringify(data) + '</div>')
    $log.append($ele);
}
